// tgDBuffer class for doing double-buffering with GDI
//
// (c) 2002 by Tom Grandgent, started on 7/7/2002

#include "tgDBuffer.h"

// Constructor
tgDBuffer::tgDBuffer(HWND hwnd)
{
	hwndWindow = hwnd;
	bufferWidth = bufferHeight = 0;
	hbmBuffer = NULL;
	hdcBuffer = NULL;
	OutputDebugString("Created double buffer\n");
}

// Destructor
tgDBuffer::~tgDBuffer(void)
{
	if (hdcBuffer)
		DeleteDC(hdcBuffer);
	if (hbmBuffer)
		DeleteObject(hbmBuffer);
}

// Returns a handle to the device context of the double buffer, used to draw onto
HDC tgDBuffer::getBuffer(RECT *clientRect)
{
	int w, h;
	HBITMAP hbmOld = NULL;

	if (!clientRect)
		GetClientRect(hwndWindow, &r);
	else
		memcpy(&r, clientRect, sizeof(RECT));
	w = r.right - r.left;
	h = r.bottom - r.top;
	// Make sure the width and height match the offscreen buffer we've been using
	if (w != bufferWidth || h != bufferHeight)
	{
		// Recreate the offscreen buffer, we've resized, or this is the first call
		if (!hdcBuffer)
			hdcBuffer = CreateCompatibleDC(NULL);
		bufferWidth = w;
		bufferHeight = h;
		if (hbmBuffer)
			hbmOld = hbmBuffer;
		hdcWindow = GetDC(hwndWindow);
		hbmBuffer = CreateCompatibleBitmap(hdcWindow, bufferWidth, bufferHeight);
		ReleaseDC(hwndWindow, hdcWindow);
		SelectObject(hdcBuffer, hbmBuffer);
		if (hbmOld)
			DeleteObject(hbmOld);	// Can't delete this bitmap until it's been selected out!
	}
	
	return hdcBuffer;
}

// Copies the contents of the double buffer to the display
void tgDBuffer::draw(void)
{
	if (!hdcBuffer)
		return;

	// Blit the offscreen buffer to the screen
	hdcWindow = GetDC(hwndWindow);
	BitBlt(hdcWindow, r.left, r.top, bufferWidth, bufferHeight, hdcBuffer, 0, 0, SRCCOPY);
	ReleaseDC(hwndWindow, hdcWindow);
}